@@ -8,21 +8,13 @@ use slings::time::delay_for;
 fn main() -> io::Result<()> {
     slings::block_on(async {
         let socket = UdpSocket::bind("127.0.0.1:0").unwrap();
-        println!("local addr: {}", socket.local_addr().unwrap());
+        println!("local addr: {}", socket.local_addr()?);
         let buf = b"helloworld";
         let addr: SocketAddr = "127.0.0.1:8081".parse().unwrap();
         loop {
-            match socket.send_to(buf, addr).await {
-                Ok(n) => {
-                    println!("send bytes: {:?}", &buf[..n]);
-                }
-                Err(e) => {
-                    println!("read fail {}", e);
-                    break;
-                }
-            }
+            let n = socket.send_to(buf, addr).await?;
+            println!("send bytes: {:?}", &buf[..n]);
             delay_for(Duration::from_secs(1)).await;
         }
-    });
-    Ok(())
+    })
 }
